/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.admin.setting;

import static de.ozgcloud.admin.setting.SettingConstants.*;

import java.util.Map;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.springframework.cloud.config.environment.Environment;
import org.springframework.cloud.config.environment.PropertySource;
import org.springframework.cloud.config.server.environment.EnvironmentRepository;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
public class SettingEnvironmentRepository implements EnvironmentRepository {

	private static final String ALFA = "Alfa";
	private static final String VORGANG_MANAGER = "OzgCloud_VorgangManager";

	private final SettingService settingService;
	private final ObjectMapper objectMapper;

	@Override
	public Environment findOne(String application, String profile, String label) {
		return buildEnvironment(application, findAnwendungSettingDTO(application));
	}

	Optional<ApplicationSettingDTO> findAnwendungSettingDTO(String application) {
		switch (application) {
			case ALFA:
				return Optional.of(settingService.getAlfaSettingDTO());
			case VORGANG_MANAGER:
				return Optional.of(settingService.getVorgangManagerSettingDTO());
			case AGGREGATION_MANAGER:
				return Optional.of(settingService.getAggregationManagerSettingDto());
			default:
				return Optional.empty();
		}
	}

	Environment buildEnvironment(String application, Optional<ApplicationSettingDTO> settingDTO) {
		var environment = new Environment(application);
		settingDTO.ifPresent(setting -> environment.add(transformToPropertySource(setting)));
		return environment;
	}

	PropertySource transformToPropertySource(ApplicationSettingDTO settingDTO) {
		return new PropertySource(StringUtils.EMPTY, objectMapper.convertValue(settingDTO, new TypeReference<Map<String, Object>>() {
		}));
	}

}
